@@ -1,6 +1,6 @@
 /* DWARF index writing support for GDB.
 
-   Copyright (C) 2018-2021 Free Software Foundation, Inc.
+   Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -32,7 +32,7 @@
    derived from INDEX_KIND is added to this base name.  DWZ_BASENAME is the
    same, but for the dwz file's index.  */
 
-extern void write_psymtabs_to_index
+extern void write_dwarf_index
   (dwarf2_per_objfile *per_objfile, const char *dir, const char *basename,
    const char *dwz_basename, dw_index_kind index_kind);
 
