@@ -1,6 +1,6 @@
 /* GDB/Scheme support for math operations on values.
 
-   Copyright (C) 2008-2019 Free Software Foundation, Inc.
+   Copyright (C) 2008-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -65,7 +65,7 @@ enum valscm_binary_opcode
 
 /* If TYPE is a reference, return the target; otherwise return TYPE.  */
 #define STRIP_REFERENCE(TYPE) \
-  ((TYPE_CODE (TYPE) == TYPE_CODE_REF) ? (TYPE_TARGET_TYPE (TYPE)) : (TYPE))
+  ((TYPE->code () == TYPE_CODE_REF) ? ((TYPE)->target_type ()) : (TYPE))
 
 /* Helper for vlscm_unop.  Contains all the code that may throw a GDB
    exception.  */
@@ -168,10 +168,10 @@ vlscm_binop_gdbthrow (enum valscm_binary_opcode opcode, SCM x, SCM y,
 	rtype = check_typedef (rtype);
 	rtype = STRIP_REFERENCE (rtype);
 
-	if (TYPE_CODE (ltype) == TYPE_CODE_PTR
+	if (ltype->code () == TYPE_CODE_PTR
 	    && is_integral_type (rtype))
 	  res_val = value_ptradd (arg1, value_as_long (arg2));
-	else if (TYPE_CODE (rtype) == TYPE_CODE_PTR
+	else if (rtype->code () == TYPE_CODE_PTR
 		 && is_integral_type (ltype))
 	  res_val = value_ptradd (arg2, value_as_long (arg1));
 	else
@@ -188,15 +188,15 @@ vlscm_binop_gdbthrow (enum valscm_binary_opcode opcode, SCM x, SCM y,
 	rtype = check_typedef (rtype);
 	rtype = STRIP_REFERENCE (rtype);
 
-	if (TYPE_CODE (ltype) == TYPE_CODE_PTR
-	    && TYPE_CODE (rtype) == TYPE_CODE_PTR)
+	if (ltype->code () == TYPE_CODE_PTR
+	    && rtype->code () == TYPE_CODE_PTR)
 	  {
 	    /* A ptrdiff_t for the target would be preferable here.  */
 	    res_val
 	      = value_from_longest (builtin_type (gdbarch)->builtin_long,
 				    value_ptrdiff (arg1, arg2));
 	  }
-	else if (TYPE_CODE (ltype) == TYPE_CODE_PTR
+	else if (ltype->code () == TYPE_CODE_PTR
 		 && is_integral_type (rtype))
 	  res_val = value_ptradd (arg1, - value_as_long (arg2));
 	else
@@ -439,7 +439,7 @@ vlscm_rich_compare (int op, SCM x, SCM y, const char *func_name)
       int result;
       switch (op)
 	{
-        case BINOP_LESS:
+	case BINOP_LESS:
 	  result = value_less (v1, v2);
 	  break;
 	case BINOP_LEQ:
@@ -451,7 +451,7 @@ vlscm_rich_compare (int op, SCM x, SCM y, const char *func_name)
 	  break;
 	case BINOP_NOTEQUAL:
 	  gdb_assert_not_reached ("not-equal not implemented");
-        case BINOP_GTR:
+	case BINOP_GTR:
 	  result = value_less (v2, v1);
 	  break;
 	case BINOP_GEQ:
@@ -524,20 +524,17 @@ vlscm_convert_typed_number (const char *func_name, int obj_arg_pos, SCM obj,
 			    int type_arg_pos, SCM type_scm, struct type *type,
 			    struct gdbarch *gdbarch, SCM *except_scmp)
 {
-  if (is_integral_type (type)
-      || TYPE_CODE (type) == TYPE_CODE_PTR)
+  if (is_integral_type (type))
     {
-      if (TYPE_UNSIGNED (type))
+      if (type->is_unsigned ())
 	{
-	  ULONGEST max;
-
-	  get_unsigned_type_max (type, &max);
+	  ULONGEST max = get_unsigned_type_max (type);
 	  if (!scm_is_unsigned_integer (obj, 0, max))
 	    {
 	      *except_scmp
-		= gdbscm_make_out_of_range_error (func_name,
-						  obj_arg_pos, obj,
-					_("value out of range for type"));
+		= gdbscm_make_out_of_range_error
+		    (func_name, obj_arg_pos, obj,
+		     _("value out of range for type"));
 	      return NULL;
 	    }
 	  return value_from_longest (type, gdbscm_scm_to_ulongest (obj));
@@ -550,15 +547,28 @@ vlscm_convert_typed_number (const char *func_name, int obj_arg_pos, SCM obj,
 	  if (!scm_is_signed_integer (obj, min, max))
 	    {
 	      *except_scmp
-		= gdbscm_make_out_of_range_error (func_name,
-						  obj_arg_pos, obj,
-					_("value out of range for type"));
+		= gdbscm_make_out_of_range_error
+		    (func_name, obj_arg_pos, obj,
+		     _("value out of range for type"));
 	      return NULL;
 	    }
 	  return value_from_longest (type, gdbscm_scm_to_longest (obj));
 	}
     }
-  else if (TYPE_CODE (type) == TYPE_CODE_FLT)
+  else if (type->code () == TYPE_CODE_PTR)
+    {
+      CORE_ADDR max = get_pointer_type_max (type);
+      if (!scm_is_unsigned_integer (obj, 0, max))
+	{
+	  *except_scmp
+	    = gdbscm_make_out_of_range_error
+	        (func_name, obj_arg_pos, obj,
+		 _("value out of range for type"));
+	  return NULL;
+	}
+      return value_from_pointer (type, gdbscm_scm_to_ulongest (obj));
+    }
+  else if (type->code () == TYPE_CODE_FLT)
     return value_from_host_double (type, scm_to_double (obj));
   else
     {
@@ -573,14 +583,13 @@ vlscm_convert_typed_number (const char *func_name, int obj_arg_pos, SCM obj,
 static int
 vlscm_integer_fits_p (SCM obj, struct type *type)
 {
-  if (TYPE_UNSIGNED (type))
+  if (type->is_unsigned ())
     {
-      ULONGEST max;
-
       /* If scm_is_unsigned_integer can't work with this type, just punt.  */
-      if (TYPE_LENGTH (type) > sizeof (scm_t_uintmax))
+      if (type->length () > sizeof (uintmax_t))
 	return 0;
-      get_unsigned_type_max (type, &max);
+
+      ULONGEST max = get_unsigned_type_max (type);
       return scm_is_unsigned_integer (obj, 0, max);
     }
   else
@@ -588,7 +597,7 @@ vlscm_integer_fits_p (SCM obj, struct type *type)
       LONGEST min, max;
 
       /* If scm_is_signed_integer can't work with this type, just punt.  */
-      if (TYPE_LENGTH (type) > sizeof (scm_t_intmax))
+      if (type->length () > sizeof (intmax_t))
 	return 0;
       get_signed_type_minmax (type, &min, &max);
       return scm_is_signed_integer (obj, min, max);
@@ -672,7 +681,7 @@ vlscm_convert_bytevector (SCM bv, struct type *type, SCM type_scm,
       make_vector_type (type);
     }
   type = check_typedef (type);
-  if (TYPE_LENGTH (type) != length)
+  if (type->length () != length)
     {
       *except_scmp = gdbscm_make_out_of_range_error (func_name, arg_pos,
 						     type_scm,
