@@ -1,6 +1,6 @@
 /* nto-tdep.h - QNX Neutrino target header.
 
-   Copyright (C) 2003-2019 Free Software Foundation, Inc.
+   Copyright (C) 2003-2022 Free Software Foundation, Inc.
 
    Contributed by QNX Software Systems Ltd.
 
@@ -26,6 +26,7 @@
 #include "osabi.h"
 #include "regset.h"
 #include "gdbthread.h"
+#include "gdbsupport/gdb-checked-static-cast.h"
 
 /* Target operations defined for Neutrino targets (<target>-nto-tdep.c).  */
 
@@ -145,17 +146,17 @@ struct nto_thread_info : public private_thread_info
 static inline nto_thread_info *
 get_nto_thread_info (thread_info *thread)
 {
-  return static_cast<nto_thread_info *> (thread->priv.get ());
+  return gdb::checked_static_cast<nto_thread_info *> (thread->priv.get ());
 }
 
 /* Per-inferior data, common for both procfs and remote.  */
 struct nto_inferior_data
 {
   /* Last stopped flags result from wait function */
-  unsigned int stopped_flags;
+  unsigned int stopped_flags = 0;
 
   /* Last known stopped PC */
-  CORE_ADDR stopped_pc;
+  CORE_ADDR stopped_pc = 0;
 };
 
 /* Generic functions in nto-tdep.c.  */
