@@ -314,11 +314,6 @@ extern void
   parse_static_tracepoint_marker_definition (const char *line, const char **pp,
 					     static_tracepoint_marker *marker);
 
-/* A hook used to notify the UI of tracepoint operations.  */
-
-extern void (*deprecated_trace_find_hook) (char *arg, int from_tty);
-extern void (*deprecated_trace_start_stop_hook) (int start, int from_tty);
-
 /* Returns the current traceframe number.  */
 extern int get_traceframe_number (void);
 
